/***************************************************************************
 *   Copyright (C) 2009 by Lassi Väätämöinen   *
 *   lassi.vaatamoinen@ixonos.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QDebug>
#include <QVariant>
#include <QColor>
#include "DownloadView.h"


DownloadView::DownloadView(QWidget* parent) :
        QTreeWidget(parent),
        items_()
{
    setRootIsDecorated(false); // Hide branch lines, making one-level treeview (similar to list)
    setHeaderItem(DownloadViewItem::getHeaderItem());

    connect(this, SIGNAL(itemPressed(QTreeWidgetItem*, int)),
            this, SLOT(on_itemClicked(QTreeWidgetItem*, int)));

}


DownloadView::~DownloadView()
{
}


void DownloadView::newItem(QTorrentHandle handle)
{
	qDebug() << "DownloadView::newItem() " << items_.count(handle);

	DownloadViewItem *item = new DownloadViewItem(QStringList()
			<< handle.name()
			<< QString::number(handle.getTotalSize())
			<< GetStatusString(handle.state())
			<< QString::number(handle.progress())
			<< QString::number(handle.downloadRate()) 
			<< QString::number(handle.uploadRate()) 
			<< QString::number(handle.numSeeds()) + "/"
					+ QString::number(handle.numLeeches())
			<< QString::number(handle.ratio())
			<< "ETA" );
	
	QBrush brushTmp(GetStatusColor(handle.state()));
	item->setForeground(2, brushTmp);
			
	addTopLevelItem(item);
	items_[handle] = item;
}


void DownloadView::updateItem(QTorrentHandle handle)
{
	qDebug() << "DownloadView::updateItem() "  << items_.count(handle);
	
	if (items_.count(handle) > 0) {
		DownloadViewItem *item = items_[handle];
		item->setData(2, Qt::DisplayRole,
									QVariant(GetStatusString(handle.state())));
		item->setData(3, Qt::DisplayRole,
									QVariant(QString::number(handle.progress())));
		item->setData(4, Qt::DisplayRole,
									QVariant(QString::number(handle.downloadRate())));
		item->setData(5, Qt::DisplayRole,
									QVariant(QString::number(handle.uploadRate())));
		item->setData(6, Qt::DisplayRole, 
									QString::number(handle.numSeeds()) + "/" 
										+ QString::number(handle.numLeeches()));
										
		QBrush brushTmp(GetStatusColor(handle.state()));
		item->setForeground(2, brushTmp);
	}

}


QTorrentHandle DownloadView::removeSelected()
{
	qDebug() << "DownloadView::removeSelected() " <<  topLevelItemCount() ;

	DownloadViewItem *item = dynamic_cast<DownloadViewItem*> (currentItem());
	
	std::map<QTorrentHandle, DownloadViewItem*>::iterator listIter
			= items_.begin();
	std::map<QTorrentHandle, DownloadViewItem*>::iterator listEnd
			= items_.end();
	
	while (listIter != listEnd) {
		if (listIter->second == item) {
			break;
		}
		++listIter;
	}
	
	QTorrentHandle handle = listIter->first;
	items_.erase(listIter);

	
	int index = indexOfTopLevelItem(currentItem());
	if (index >= 0) {
		takeTopLevelItem(index);
	}
	
	qDebug() << "DownloadView::removeSelected() " <<  topLevelItemCount() ;
	
	return handle;
}


void DownloadView::removeItem(QTorrentHandle handle)
{
}


void DownloadView::on_itemClicked(QTreeWidgetItem * item, int column)
{
	/*
	qDebug() << "DownloadView::on_itemClicked(()" << item << "," << column;
	qDebug() << "current item" << currentItem();
	
	if (item == currentItem() && item->isSelected()) {
		item->setSelected(false);
	}
	*/
}


QString DownloadView::GetStatusString(QTorrentHandle::State const& status) const
{
	switch (status) {
		case QTorrentHandle::QUEUED_FOR_CHECKING :
			return tr("Queued");
		case QTorrentHandle::CHECKING_FILES :
			return tr("Checking");
		case QTorrentHandle::DOWNLOADING_METADATA :
			return tr("DL meta");
		case QTorrentHandle::DOWNLOADING :
			return tr("Downloading");
		case QTorrentHandle::FINISHED :
			return tr("Finished");
		case QTorrentHandle::SEEDING :
			return tr("Seeding"); 
		case QTorrentHandle::ALLOCATING :
			return tr("Allocating");
		default:
			return tr("N/A");
	}
}


QColor DownloadView::GetStatusColor(QTorrentHandle::State const& status) const
{
	QColor green(40,205,40);
	QColor yellow(255,174,0);

	switch (status) {
		case QTorrentHandle::QUEUED_FOR_CHECKING :
		case QTorrentHandle::CHECKING_FILES :
		case QTorrentHandle::DOWNLOADING_METADATA :
		case QTorrentHandle::ALLOCATING :
			return yellow;
		case QTorrentHandle::DOWNLOADING :
		case QTorrentHandle::FINISHED :
		case QTorrentHandle::SEEDING :
				return green;
		default:
			return QColor();
	}
}